//----------------------------------------------------------------------------
// SVG Viewer - Version 1.0
// A library of SVG renderer
// Copyright (C) 2008 Witold Jaworski
// Contact:	witold.jaworski@airplanes3d.net
//			http://airplanes3d.net
// 
// Contains elements from:
//			AGG - a graphic library (Copyright Maxim Shemanarev, mcseem@antigrain.com)
//          http://antigrain.com
// Uses functions from:
//          Expat - a XML parser library (Copyright James Clark)
// 
// SVG Viewer is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// SVG Viewer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with SVG Viewer; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
// MA 02110-1301, USA.
//----------------------------------------------------------------------------
#include "svg_exception.h"
#include "svg_conv_locator.h"

namespace svg
{
	//Advances the current position to the begining of next 
	//segment of the curve.
	//Arguments:
	//	broken: true, when there was a gap before the selected
	//	segment.
	//Returns false, when the last segment has been reached.
	bool conv_locator::next_segment(bool &broken)
	{
		double xn, yn, x0, y0;

		//defaults:	
		broken = false;

		m_spos += m_a.length(); //add the whole length of actual segment
								//m_spos now points to the begin of the
								//next segment

		if (m_continue == is_closed) //it was the last segment of the closed
		{							 //path!
			x0 = m_a.x(); y0 = m_a.y(); //end of actual segment
			m_continue = is_terminated;
			if (m_size < 0.0) m_size = m_spos;
			m_spos = 0.0;
			m_a = segment(x0,y0,0.0,m_a.dx(), m_a.dy()); //set up the initial segment of the path!
			return false;			
		}
		//Take another segment end:
		unsigned cmd = m_vs.vertex(&xn, &yn);

		if(agg::is_close(cmd) || agg::is_stop(cmd)) 
		{
			x0 = m_a.x(); y0 = m_a.y(); //end of actual segment
			
			if (agg::is_close(cmd))//closed path!
			{
				m_vs.rewind(m_path_id); //we have to rewind this line!
				if (!agg::is_vertex(cmd = m_vs.vertex(&xn,&yn)))
				{
					char msg[128];
					sprintf(msg,"conv_locator::next_segment: Expected: path_cmd_move_to. Unexpected vertex command: %x.", cmd);
					throw new exception(msg);
				}
				m_a = segment(x0, y0, xn, yn);
				m_continue = is_closed;
			}
			else
			{
				if (m_size < 0.0) m_size = m_spos;
				m_a = segment(x0, y0);
				m_continue = is_terminated;
			}

			m_l0 = 0.0;
		}
		else
		{

			if (agg::is_line_to(cmd))
			{
				x0 = m_a.x(); y0 = m_a.y(); //end of actual segment
				m_a = segment(x0, y0, xn, yn); //new segment: end of the 
											   //previous, and the new vertex
				m_l0 = 0.0; //initial position: start
			}
			else
			{
				if(agg::is_move_to(cmd))
				{
					m_a = segment(xn, yn);
					broken = true;
					return next_segment(); //it will set us at the begining
										   // of the (xn,yn, xn+1, yn+1)	
				}
				else
				{
					char msg[128];
					sprintf(msg,"conv_locator::next_segment: Unknown vertex command: %x.", cmd);
					throw new exception(msg);
				}//if is move to
			}//if is line to
		}//if is stop
		return m_continue != is_terminated;
	}

	//Advances actual position by given distance
	//Arguments:
	//	span:	length that should be advanced
	//Returns false, when the end of the curve has been reached
	//(in this case most probably the real distance is shorter)
	bool conv_locator::move(double span)
	{
		double end = position() + span;  //desired position

		if (end < 0.0) end = 0.0; //clip the result within path!

		while (
				((m_spos + m_a.length()) < end)	//the end is ahead
					&&							//of actual segment?	
				! terminated()					//.. and curve has not finished!
			  )
		{
			next_segment(); //sets m_l0 = 0.0
		}//end while

		//the point is inside current segment, 
		//(of course, if the curve has not been terminated, yet!)
		if (terminated())
		{
			m_l0 = m_a.length();
			return false;
		}
		else
		{
			m_l0 = (end - m_spos);
			return true;
		}
	}

} // namespace svg
