//----------------------------------------------------------------------------
// SVG Viewer - Version 1.0
// A library of SVG renderer
// Copyright (C) 2008 Witold Jaworski
// Contact:	witold.jaworski@airplanes3d.net
//			http://airplanes3d.net
// 
// Contains elements from:
//			AGG - a graphic library (Copyright Maxim Shemanarev, mcseem@antigrain.com)
//          http://antigrain.com
// Uses functions from:
//          Expat - a XML parser library (Copyright James Clark)
// 
// SVG Viewer is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// SVG Viewer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with SVG Viewer; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
// MA 02110-1301, USA.
//----------------------------------------------------------------------------

#ifndef SVG_EARRAY_INCLUDED
#include "svg_element.h"
#include "svg_req_render.h"
#define SVG_EARRAY_INCLUDED

namespace svg
{
//////////////////////////////////////////////////////////////////////
// earray: a helper element (non-existent in the source file) to store
//		   an array of arbitrary lengths (spans). Used for the 
//		   texts - their attributes x, y, dx, dy, rotation can contain
//		   many values, separated by commas or spaces
//////////////////////////////////////////////////////////////////////
class earray : public element 
{
	public:
		//////////////////////////////////////////////////////////////////////
		// constructor and overriden virtual methods
		//////////////////////////////////////////////////////////////////////

		earray(element *parent, unsigned level): element(parent, level) {}

		//Static constructor - used by the parser to create a new dash pattern
		//Arguments:
		//	prs:	parser;
		//	parent:	the element, that will be the parent of this element
		//	level:	the level of nesting in the source XML tree (just parent->level()+1 here)
		//	spec:	the specification of this element - a string, which plays both roles:
		//			the complete specification of the object content, and its name.
		static element *create(parser &prs, element *parent, unsigned level, const char *spec)
		{
			earray *elm = new earray(parent, level);
			char span[SVG_MAX_NAME]; //a buffer for the single span length...
			char separator = strchr(spec,',')?',':' '; //check: comma or space?
			char *p = (char *) spec;
			while (*p)
			{
				p = prs.get_first_token(p,separator,span,sizeof(span));
				elm->on_attribute(prs,"span",span);
			}

			//copy the specification - it will be the name...
			if (strlen(spec) < sizeof(elm->m_spec))
				strncpy(elm->m_spec,spec,sizeof(elm->m_spec));
			else
				elm->m_spec[0] = 0; //or just an empty string, when it is too long

			elm->m_name = elm->m_spec;

			return elm;
		}

		//prefix "ext:" is for elements that have no corresponding element in 
		//the XML SVG document structure
		virtual const char * tag() {return "ext:earray";}; 
														 

		virtual void on_attribute(parser &prs, const char * name, const char *value)
		{
			if (strcmp(name, "span") == 0) //there is no such attribute in the XML 
			{							   //see parser::parse_style() function	
				 prs.store(*this,attribute(attribute::dash_span,prs.to_length(value)));
			}
			else 
			{
				//nothing - this element has not to have any other attributes
			}
		}

		
	public:
		//////////////////////////////////////////////////////////////////////
		// Other methods and fields
		//////////////////////////////////////////////////////////////////////
		//Returns the nth value from the sequence
		//Arguments
		//	req:	any request
		//	n:		the index of the value 
		//	deflt:	default value - returned, when the requested index 
		//			is outside the number of the elements contained
		double get(request &req, unsigned n, double deflt = 0.0)
		{
			if (m_attribute_first == invalid_handle) return deflt;
			if ((m_attribute_first + n) > m_attribute_last) return deflt;

			attribute *span = req.get_attribute(m_attribute_first + n);
			if (span) return span->val.real;
			else return deflt;
		}
		
		//Returns number of the elements in this array
		unsigned count(void) 
		{
			if (m_attribute_first == invalid_handle)
			{
				return 0;
			}
			else
			{
				return m_attribute_last - m_attribute_first + 1;
			}
		}

	protected:
		char m_spec[SVG_MAX_ATTRIB_VALUE];
}; //earray

}; //namespace svg
#endif //SVG_EARRAY_INCLUDED

