//----------------------------------------------------------------------------
// SVG Viewer - Version 1.0
// A library of SVG renderer
// Copyright (C) 2008 Witold Jaworski
// Contact:	witold.jaworski@airplanes3d.net
//			http://airplanes3d.net
// 
// Contains elements from:
//			AGG - a graphic library (Copyright Maxim Shemanarev, mcseem@antigrain.com)
//          http://antigrain.com
// Uses functions from:
//          Expat - a XML parser library (Copyright James Clark)
// 
// SVG Viewer is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// SVG Viewer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with SVG Viewer; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
// MA 02110-1301, USA.
//----------------------------------------------------------------------------
#include "svg_exception.h"
#include "svg_element.h"

namespace svg
{
	//Handles the request, and forwards it to the descendants.
	//Arguments:
	//	req:	the request instance - usually with helper data.
	//			Use the req.type() to deterimine, what is it,
	//			and what class it should be cated to.
	//Returns the first id of not rendered SVG graph element
	//Default implementation just forwards the request down the SVG graph.
	//Rememeber, that each "return" in the overrides should return the
	//result of forward() or skip() functions! Otherwise you can have some
	//problems!
	handle element::on_request(request &req) 
	{
#ifdef TEX_EXTENSIONS
		switch(req.type())
		{
		case request::override:
			{
				req_override & rovr =  (req_override &) req;

				handle aid = invalid_handle;
				if(is_valid(m_attribute_last))
				{
					for (aid = m_attribute_first; aid <= m_attribute_last; aid++)
					{
						attribute *attr = req.get_attribute(aid);
						if (attr && attr->type == rovr.value.type) 
						{
							*attr = rovr.value;
						}//end if
					}//end for
				}

				if (rovr.propagate)	return forward(req);
				else return skip(req);

			}//end case	

		default:
			return forward(req);
		}
#else
		return forward(req);
#endif //TEX_EXTENSIONS
	}

//----------------------------------------------------------------------------
	//Called just after the element constructor: normally it should analyze and -
	//optionally - store the recognized attribute
	//Arguments:
	//	prs:	parser, that actually prepares the whole thing
	//	name:	name of the attribute;
	//	value:	value of the attribute (text);
	void element::on_attribute(parser &prs, const char * name, const char *value)
	{
		if (strcmp(name, "id") == 0) //We have got the unique element's name (xml id)
		{	
			//important lines - presistence of a new element:
			bool defs = (parent() && (strcmp(parent()->tag(), "defs") == 0));
			//defs: when true, adds also reference to *this to the lookup table
			//which allows other elements to access it in a quicker way
			prs.store(*this,value,defs); //stores *this into the element's storage
		}
	} //element::on_attribute

//----------------------------------------------------------------------------
	//Returns an id of element that has given name
	//	req:			actual request instance
	//	type:			the type of the attribute
	//	search_from:	optional - the handle id that should be used
	//					Usin this parameter, you can find another
	//					occurrence of an attribute of given type.
	//Remark:	returns an invalid_handle constant, when not found
	handle element::get_id(request &req, attribute::kind type, handle search_from)
	{
		handle id;
		handle start = (search_from == invalid_handle? m_attribute_first : search_from);
		handle end = m_attribute_last;
		attribute * a;
		for (id= start; id <= end; id++ )
		{
			if ((a = req.get_attribute(id)) && (a->type == type)) break;
				
		}
		return (id >= end) ? invalid_handle : id;
	}


} // namespace svg
